@@ -14,8 +14,6 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
-#define _GNU_SOURCE
-
 #include <stdio.h>
 #include <stdarg.h>
 #include <ctype.h>
@@ -223,17 +221,14 @@ void dotnet_parse_us(
   {
     blob_result = dotnet_parse_blob_entry(pe, offset);
 
-    if (blob_result.size == 0 || !fits_in_pe(pe, offset, blob_result.length))
-    {
-      set_integer(i, pe->object, "number_of_user_strings");
-      return;
-    }
+    if (blob_result.size == 0)
+      break;
 
     offset += blob_result.size;
     // Avoid empty strings, which usually happen as padding at the end of the
     // stream.
 
-    if (blob_result.length > 0)
+    if (blob_result.length > 0 && fits_in_pe(pe, offset, blob_result.length))
     {
       set_sized_string(
          (char*) offset,
@@ -351,6 +346,7 @@ void dotnet_parse_tilde_2(
   PMODULE_TABLE module_table;
   PASSEMBLY_TABLE assembly_table;
   PASSEMBLYREF_TABLE assemblyref_table;
+  PFIELDRVA_TABLE fieldrva_table;
   PMANIFESTRESOURCE_TABLE manifestresource_table;
   PMODULEREF_TABLE moduleref_table;
   PCUSTOMATTRIBUTE_TABLE customattribute_table;
@@ -363,7 +359,7 @@ void dotnet_parse_tilde_2(
   int bit_check;
   int matched_bits = 0;
 
-  int64_t resource_offset;
+  int64_t resource_offset, field_offset;
   uint32_t row_size, row_count, counter;
 
   const uint8_t* string_offset;
@@ -1049,7 +1045,30 @@ void dotnet_parse_tilde_2(
         break;
 
       case BIT_FIELDRVA:
-        table_offset += (4 + index_sizes.field) * num_rows;
+        row_size = 4 + index_sizes.field;
+        row_ptr = table_offset;
+
+        // Can't use 'i' here because we only set the field offset if it is
+        // valid. Instead use 'counter'.
+        counter = 0;
+
+        for (i = 0; i < num_rows; i++)
+        {
+          fieldrva_table = (PFIELDRVA_TABLE) row_ptr;
+
+          field_offset = pe_rva_to_offset(pe, fieldrva_table->RVA);
+          if (field_offset >= 0)
+          {
+            set_integer(field_offset, pe->object, "field_offsets[%i]", counter);
+            counter++;
+          }
+
+          row_ptr += row_size;
+        }
+
+        set_integer(counter, pe->object, "number_of_field_offsets");
+
+        table_offset += row_size * num_rows;
         break;
 
       case BIT_ENCLOG:
@@ -1140,12 +1159,15 @@ void dotnet_parse_tilde_2(
         break;
 
       case BIT_ASSEMBLYREF:
-        row_size = (2 + 2 + 2 + 2 + 4 + (index_sizes.blob * 2) + (index_sizes.string * 2));
+        row_size = (2 + 2 + 2 + 2 + 4 +
+            (index_sizes.blob * 2) +
+            (index_sizes.string * 2));
+
         row_ptr = table_offset;
 
         for (i = 0; i < num_rows; i++)
         {
-          if (!fits_in_pe(pe, table_offset, row_size))
+          if (!fits_in_pe(pe, row_ptr, row_size))
             break;
 
           assemblyref_table = (PASSEMBLYREF_TABLE) row_ptr;
@@ -1169,6 +1191,7 @@ void dotnet_parse_tilde_2(
                 assemblyref_table->PublicKeyOrToken.PublicKeyOrToken_Short;
 
           blob_result = dotnet_parse_blob_entry(pe, blob_offset);
+          blob_offset += blob_result.size;
 
           if (blob_result.size == 0 ||
               !fits_in_pe(pe, blob_offset, blob_result.length))
@@ -1180,7 +1203,6 @@ void dotnet_parse_tilde_2(
           // Avoid empty strings.
           if (blob_result.length > 0)
           {
-            blob_offset += blob_result.size;
             set_sized_string((char*) blob_offset,
                 blob_result.length, pe->object,
                 "assembly_refs[%i].public_key_or_token", i);
@@ -1663,6 +1685,9 @@ begin_declarations;
   declare_string_array("constants");
   declare_integer("number_of_constants");
 
+  declare_integer_array("field_offsets");
+  declare_integer("number_of_field_offsets");
+
 end_declarations;
 
 
