@@ -355,6 +355,26 @@ static void test_syntax()
   assert_error(
       "rule test { strings: $a = \"a\" $a = \"a\" condition: all of them }",
       ERROR_DUPLICATED_STRING_IDENTIFIER);
+
+  assert_error(
+      "rule test { strings: $a = /a.c/ xor condition: $a }",
+      ERROR_SYNTAX_ERROR);
+
+  assert_error(
+      "rule test { strings: $a = /abc/ xor condition: $a }",
+      ERROR_SYNTAX_ERROR);
+
+  assert_error(
+      "rule test { strings: $a = {01 02 ?? 03 04} xor condition: $a }",
+      ERROR_SYNTAX_ERROR);
+
+  assert_error(
+      "rule test { strings: $a = {01 02 0? 03 04} xor condition: $a }",
+      ERROR_SYNTAX_ERROR);
+
+  assert_error(
+      "rule test { strings: $a = {01 02 03 04} xor condition: $a }",
+      ERROR_SYNTAX_ERROR);
 }
 
 
@@ -509,14 +529,16 @@ static void test_strings()
              all of them\n\
        }", "abcdef");
 
+  // xor by itself will match the plaintext version of the string too.
   assert_true_rule_file(
     "rule test {\n\
       strings:\n\
         $a = \"This program cannot\" xor\n\
       condition:\n\
-        #a == 255\n\
+        #a == 256\n\
     }", "tests/data/xor.out");
 
+  // Make sure the combination of xor and ascii behaves the same as just xor.
   assert_true_rule_file(
     "rule test {\n\
       strings:\n\
@@ -525,6 +547,25 @@ static void test_strings()
         #a == 256\n\
     }", "tests/data/xor.out");
 
+  assert_true_rule_file(
+    "rule test {\n\
+      strings:\n\
+        $a = \"This program cannot\" xor(1-0x10)\n\
+      condition:\n\
+        #a == 16\n\
+    }", "tests/data/xor.out");
+
+  // We should have no matches here because we are not generating the ascii
+  // string, just the wide one, and the test data contains no wide strings.
+  assert_true_rule_file(
+    "rule test {\n\
+      strings:\n\
+        $a = \"This program cannot\" xor wide\n\
+      condition:\n\
+        #a == 0\n\
+    }", "tests/data/xor.out");
+
+  // xor by itself is equivalent to xor(0-255).
   assert_true_rule_file(
     "rule test {\n\
       strings:\n\
@@ -533,13 +574,127 @@ static void test_strings()
         #a == 256\n\
     }", "tests/data/xorwide.out");
 
+  // This DOES NOT look for the plaintext wide version by itself.
   assert_true_rule_file(
+    "rule test {\n\
+      strings:\n\
+        $a = \"This program cannot\" xor(1-16) wide\n\
+      condition:\n\
+        #a == 16\n\
+    }", "tests/data/xorwide.out");
+
+  // Check the location of the match to make sure we match on the correct one.
+  assert_true_rule_file(
+    "rule test {\n\
+      strings:\n\
+        $a = \"This program cannot\" xor(1) wide\n\
+      condition:\n\
+        #a == 1 and @a == 0x2f\n\
+    }", "tests/data/xorwide.out");
+
+  assert_error(
+    "rule test {\n\
+      strings:\n\
+        $a = \"This program cannot\" xor(300)\n\
+      condition:\n\
+        $a\n\
+    }", ERROR_INVALID_MODIFIER);
+
+  assert_error(
+    "rule test {\n\
+      strings:\n\
+        $a = \"This program cannot\" xor(200-10)\n\
+      condition:\n\
+        $a\n\
+    }", ERROR_INVALID_MODIFIER);
+
+  assert_error(
+    "rule test {\n\
+      strings:\n\
+        $a = {00 11 22 33} xor\n\
+      condition:\n\
+        $a\n\
+    }", ERROR_SYNTAX_ERROR);
+
+  assert_error(
+    "rule test {\n\
+      strings:\n\
+        $a = /foo(bar|baz)/ xor\n\
+      condition:\n\
+        $a\n\
+    }", ERROR_SYNTAX_ERROR);
+
+  assert_error(
+    "rule test {\n\
+      strings:\n\
+        $a = \"ab\" xor xor\n\
+      condition:\n\
+        $a\n\
+    }", ERROR_DUPLICATED_MODIFIER);
+
+  // We should have no matches here because we are not generating the wide
+  // string, just the ascii one, and the test data contains no ascii strings.
+  assert_true_rule_file(
+    "rule test {\n\
+      strings:\n\
+        $a = \"This program cannot\" xor ascii\n\
+      condition:\n\
+        #a == 0\n\
+    }", "tests/data/xorwide.out");
+
+  // This should match 512 times because we are looking for the wide and ascii
+  // versions in plaintext and doing xor(0-255) (implicitly)
+  assert_true_rule_file(
+    "rule test {\n\
+      strings:\n\
+        $a = \"This program cannot\" xor wide ascii\n\
+      condition:\n\
+        #a == 512\n\
+    }", "tests/data/xorwideandascii.out");
+
+  assert_true_rule_file(
+    "rule test {\n\
+      strings:\n\
+        $a = \"This program cannot\" wide ascii\n\
+      condition:\n\
+        #a == 2\n\
+    }", "tests/data/xorwideandascii.out");
+
+  assert_error(
     "rule test {\n\
       strings:\n\
         $a = \"ab\" xor nocase\n\
       condition:\n\
-        #a == 1084\n\
-    }", "tests/data/xornocase.out");
+        true\n\
+    }", ERROR_INVALID_MODIFIER);
+
+  assert_true_rule(
+      "rule test { \
+        strings:\
+          $a = \"AXS\" private\
+      condition:\
+        all of them\
+      }",
+      "AXS");
+
+  assert_true_rule(
+      "rule test { \
+        strings:\
+          $a = { 45 52 53 } private\
+      condition:\
+        all of them\
+      }",
+      "ERS");
+
+  assert_true_rule(
+      "rule test { \
+        strings:\
+          $a = /AXS[0-9]{4}ERS[0-9]{4}/ private\
+      condition:\
+        all of them\
+      }",
+      "AXS1111ERS2222");
+
 }
 
 
@@ -638,6 +793,13 @@ static void test_hex_strings()
         condition: $a }",
       "1234567890");
 
+  assert_true_rule(
+      "rule test { \
+        strings: $a = { 31 32 [-] 33 34 [-] 38 39 } private \
+        condition: $a }",
+      "1234567890");
+
+
   assert_true_rule(
       "rule test { \
         strings: $a = { 31 32 [1] 34 35 [2] 38 39 } \
@@ -784,6 +946,10 @@ static void test_count()
   assert_true_rule(
       "rule test { strings: $a = \"ssi\" condition: #a == 2 }",
       "mississippi");
+
+  assert_true_rule(
+      "rule test { strings: $a = \"ssi\" private condition: #a == 2 }",
+      "mississippi");
 }
 
 
@@ -795,6 +961,12 @@ static void test_at()
         condition: $a at 2 and $a at 5 }",
       "mississippi");
 
+  assert_true_rule(
+      "rule test { \
+        strings: $a = \"ssi\" private \
+        condition: $a at 2 and $a at 5 }",
+      "mississippi");
+
   assert_true_rule(
       "rule test { \
         strings: $a = \"mis\" \
@@ -816,6 +988,12 @@ static void test_in()
         strings: $a = { 6a 2a 58 c3 } \
         condition: $a in (entrypoint .. entrypoint + 1) }",
       PE32_FILE);
+
+  assert_true_rule_blob(
+      "rule test { \
+        strings: $a = { 6a 2a 58 c3 } private \
+        condition: $a in (entrypoint .. entrypoint + 1) }",
+      PE32_FILE);
 }
 
 
@@ -825,6 +1003,10 @@ static void test_offset()
       "rule test { strings: $a = \"ssi\" condition: @a == 2 }",
       "mississippi");
 
+  assert_true_rule(
+      "rule test { strings: $a = \"ssi\" private condition: @a == 2 }",
+      "mississippi");
+
   assert_true_rule(
       "rule test { strings: $a = \"ssi\" condition: @a == @a[1] }",
       "mississippi");
@@ -841,6 +1023,10 @@ static void test_length()
       "rule test { strings: $a = /m.*?ssi/ condition: !a == 5 }",
       "mississippi");
 
+  assert_true_rule(
+      "rule test { strings: $a = /m.*?ssi/ private condition: !a == 5 }",
+      "mississippi");
+
   assert_true_rule(
       "rule test { strings: $a = /m.*?ssi/ condition: !a[1] == 5 }",
       "mississippi");
@@ -887,7 +1073,7 @@ static void test_of()
       "mississippi");
 
   assert_true_rule(
-      "rule test { strings: $a = \"ssi\" $b = \"mis\" $c = \"oops\" "
+      "rule test { strings: $a = \"ssi\" $b = \"mis\" private $c = \"oops\" "
       "condition: 1 of them }",
       "mississippi");
 
@@ -962,6 +1148,27 @@ void test_for()
           for all i in (1..#a) : (@a[i] == 5) \
       }",
       "mississippi");
+
+  assert_true_rule(
+      "rule test { \
+        condition: \
+          for any i in (1, 2, 3) : (i <= 1) \
+      }",
+      NULL);
+
+  assert_true_rule(
+      "rule test { \
+        condition: \
+          for all i in (1, 2, 3) : (i >= 1) \
+      }",
+      NULL);
+
+  assert_false_rule(
+      "rule test { \
+        condition: \
+          for all i in (1, 0) : (i != 1) \
+      }",
+      NULL);
 }
 
 
@@ -1234,12 +1441,16 @@ void test_re()
   assert_true_regexp("a\\sb", "a\nb", "a\nb");
   assert_true_regexp("a\\sb", "a\vb", "a\vb");
   assert_true_regexp("a\\sb", "a\fb", "a\fb");
+  assert_true_regexp("a[\\s]*b", "a \t\r\n\v\fb", "a \t\r\n\v\fb");
+  assert_true_regexp("a[^\\S]*b", "a \t\r\n\v\fb", "a \t\r\n\v\fb");
   assert_false_regexp("a\\Sb", "a b");
   assert_false_regexp("a\\Sb", "a\tb");
   assert_false_regexp("a\\Sb", "a\rb");
   assert_false_regexp("a\\Sb", "a\nb");
   assert_false_regexp("a\\Sb", "a\vb");
   assert_false_regexp("a\\Sb", "a\fb");
+  assert_true_regexp("foo([^\\s]*)", "foobar\n", "foobar");
+  assert_true_regexp("foo([^\\s]*)", "foobar\r\n", "foobar");
   assert_true_regexp("\\n\\r\\t\\f\\a", "\n\r\t\f\a", "\n\r\t\f\a");
   assert_true_regexp("[\\n][\\r][\\t][\\f][\\a]", "\n\r\t\f\a", "\n\r\t\f\a");
   assert_true_regexp("\\x01\\x02\\x03", "\x01\x02\x03", "\x01\x02\x03");
@@ -1410,6 +1621,11 @@ void test_re()
   assert_false_rule_blob(
        "rule test { strings: $a = \" cmd.exe \" nocase wide condition: $a }",
        ISSUE_1006);
+
+  // Test case for issue #1117
+  assert_true_rule_blob(
+       "rule test { strings: $a =/abc([^\"\\\\])*\"/ nocase condition: $a }",
+       "abc\xE0\x22");
 }
 
 
@@ -1803,6 +2019,10 @@ static void test_hash_module()
             and \
           hash.sha256(1, filesize) == \
             \"aaaaf2863e043b9df604158ad5c16ff1adaf3fd7e9fcea5dcb322b6762b3b59a\" \
+            and \
+          hash.crc32(0, filesize) == 0x8587d865 \
+            and \
+          hash.crc32(\"TEST STRING\") == 0x51f9be31 \
       }",
       blob);
 
@@ -1896,6 +2116,7 @@ void test_process_scan()
         all of them\
     }", &rules) == ERROR_SUCCESS);
   rc1 = yr_rules_scan_proc(rules, pid, 0, count_matches, &matches, 0);
+  yr_rules_destroy(rules);
   kill(pid, SIGALRM);
 
   rc2 = waitpid(pid, &status, 0);
@@ -2110,6 +2331,5 @@ int main(int argc, char** argv)
   test_performance_warnings();
 
   yr_finalize();
-
   return 0;
 }
